/*
 * main.cpp
 *
 *  Created on: Jan 22, 2009
 *      Author: jonathanmasci
 */

#include "huffman.h"
#include <fstream>

int main(int argc, char **argv)
{
  int estimation = 0;
  int length;
  int symlen = 0;
  float entropy = 0;
  bool verbose = false;
  std::ifstream in;
  if (strcmp(argv[1], "-v") == 0)
    {
      in.open(argv[2], std::ifstream::binary);
      symlen = atoi(argv[3]);
      verbose = true;
    }
  else
    {
      in.open(argv[1], std::ifstream::binary);
      symlen = atoi(argv[2]);
    }
  // get length of file:
  in.seekg (0, std::ios::end);
  length = in.tellg();
  in.seekg (0, std::ios::beg);
  // allocate memory:
  char *input = new char[length];
  // read data as a block:
  in.read(input,length);
  in.close();

  huffman *huf = new huffman(input, length, symlen);
  huf->computeWeights();
  printf("Done weights\n");
  if (verbose)
    huf->printWeights();
  huf->buildTree();
  printf("Done tree\n");
  huf->buildCodeTable();
  printf("Done codetable\n");
  if (verbose)
    huf->printCodeTable();
  estimation = huf->getBitSizeEstimation();
  printf("#symbols = %i, Compressed estimation = %i\nDictionary size = %i\n",
      huf->getSymbolsCount(),
      estimation,
      huf->getDictionarySize());
  entropy = huf->getEntropy();
  printf("Entropy = %f\n", entropy);
  printf("Weighted path length = %f\n", huf->getWeightedPathLength());
  delete[] input;

  return 0;
}
